
# THIS FILE IS GENERATED FROM UWGeodynamics SETUP.PY
#
underworld_version = 'NOT INSTALLED'
short_version = '0.9.2'
version = '0.9.2'
full_version = '0.9.2-dev-10ac92e(master)'
git_revision = '10ac92ee87a6139b5a1e4ed5f11be23651103025'
release = False

if not release:
    version = full_version
