
# List of Rheology setups
Parameters = {}

Parameters["Resolution"] = [(256,128),
                            (512,256),
                            (1024,512),
                            (2048,1024)]

Parameters["Average Method"] = ["arithmetic",
                                "geometric",
                                "harmonic",
                                "maximum",
                                "minimum",
                                "root mean square"]
